package audiotesting;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;

/**
 *Clase encargada de asoicar laa componentes del circuito o diagrama a realizar
 */
public class SoundCircuit {
    private JPanel panel;
    private Simulator simulator;
    private ArrayList<Drawable> components;
    
    /**
     * Constructor
     */
    public SoundCircuit(){
        simulator = new Simulator();
        components = new ArrayList<Drawable>();
    }

    /**
     * Setea el panel
     * @param p 
     */
    public void setPanel(JPanel p){
        panel = p;
    }
    
    /**
     * Simula el circuito
     */
    public void simulate(){
      simulator.simulate();
    }
    
    /**
     * Busca el drawable asociado al punto
     * @param p
     * @return 
     */
    public Drawable find(Point p) {
        for (Drawable component : components)
            if (component.contains(p))
                return component;
        return null;
    }
    
    /**
     * Encuantra la salida asociada al shape
     * @param rec
     * @return 
     */
    public GOutSignal findSource(Rectangle2D rec) {
       for (Drawable component : components) 
         if (component instanceof GOutSignal) {
            GOutSignal source = (GOutSignal)component;
            if (source.getGOutput().intersects(rec))
              return source;
         }
       return null;
    }
    
    /**
     * Borra la fuente
     * @param Source 
     */
    public void deleteSource(Drawable Source) {
        components.remove(Source);
    }
  
    /**
     * Encuatra las entradas asocadas a una shape
     * @param rec
     * @return 
     */
    public GInSignal findDrain(Rectangle2D rec) {
       for (Drawable component : components) 
         if (component instanceof GInSignal) {
            GInSignal drain = (GInSignal)component;
            if (drain.getGInput().intersects(rec))
              return drain;
         }
       return null;
    }
    
    //Dibuja todos los bloques
    public void paintComponent(Graphics2D g) {
        for (Drawable c: components) 
            c.draw(g);
    }
    
    /**
     * Agrega el Filtro Pasabanda
     * @param position_x
     * @param position_y 
     */
    public void addFiltroPasabanda(double position_x, double position_y) {
        components.add(new GFiltroPasabanda(new Point2D.Double(position_x, position_y)));
        panel.repaint();
    }
     
    /**
     * Metodo encargado de agragar un pasa bajo
     * @param x
     * @param y 
     */
     public void addFiltroPasabajo(double x, double y) {
        components.add(new GFiltroPasabajo(new Point2D.Double(x,y)));
        panel.repaint();
    }
     
     /**
      * Agrega filtro pasa alto
      * @param x
      * @param y 
      */
     public void addFiltroPasaalto(double x, double y) {
        components.add(new GFiltroPasaalto(new Point2D.Double(x,y)));
        panel.repaint();
    }
    
     /**
      * Agrega efecto Flanger
      * @param position_x
      * @param position_y 
      */
    public void addFlanger(double position_x, double position_y) {
        components.add(new GFlanger(new Point2D.Double(position_x, position_y)));
        panel.repaint();
    }
    
    /**
     * Agrega el efecto Delay
     * @param position_x
     * @param position_y 
     */
    public void addDelay(double position_x, double position_y) {
        components.add(new GDelay(new Point2D.Double(position_x, position_y)));
        panel.repaint();
    }
    
    /**
     * Agregar wire
     * @return 
     */
    public GWire addWire(double x, double y) {
        GWire actualWire = new GWire(new Point2D.Double(x,y), this); 
        components.add(actualWire);
        panel.repaint();
        return actualWire;
    }
    
    /**
     * Agrega el audio de entrada
     */
    public void addAudio(double x, double y) {
        components.add(new GAudioEntrada(new Point2D.Double(x,y), simulator));
        panel.repaint();
    }
    
    /**
     * Agrega al audio de salida
     */
    public void addOutSound(double x, double y) {
        components.add(new GAudioSalida(new Point2D.Double(x,y), simulator));
        panel.repaint();
    }
    
    /**
     * Escucha el audio de la salida
     */
    public void escuchar() {
        simulator.escuchar();
    }
    
    /**
     * retorna el paner correspondiente al circuito de sonido
     * @return 
     */
    public JPanel getpanel(){
        return panel;
        
    }
}

